/*******************************************************************************
 *
 * blueMarine - open source photo workflow
 * =======================================
 *
 * Copyright (C) 2003-2010 by Fabrizio Giudici
 * Project home page: http://bluemarine.tidalwave.it
 *
 *******************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *******************************************************************************
 *
 * $Id: ProgressInputStream.java,v 213fe927a495 2010/01/01 00:36:29 fabrizio $
 *
 ******************************************************************************/
package it.tidalwave.semantic.importexport;

import javax.annotation.Nonnegative;
import javax.annotation.Nonnull;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;
import java.awt.EventQueue;
import it.tidalwave.util.logging.Logger;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: ProgressInputStream.java,v 213fe927a495 2010/01/01 00:36:29 fabrizio $
 *
 ******************************************************************************/
public class ProgressInputStream extends InputStream
  {
    private final static String CLASS = ProgressInputStream.class.getName();
    private final static Logger logger = Logger.getLogger(CLASS);

    public static final String PROP_TOTAL_BYTE_COUNT = "totalByteCount";
    public static final String PROP_PROCESSED_BYTE_COUNT = "processedByteCount";
    
    @Nonnull
    private final InputStream delegate;

    @Nonnegative
    protected int totalByteCount;

    @Nonnegative
    protected int processedByteCount;

    private final PropertyChangeSupport propertyChangeSupport = new PropertyChangeSupport(this);

    /***************************************************************************
     *
     *
     **************************************************************************/
    public ProgressInputStream (final File file)
      throws FileNotFoundException
      {
        this(new FileInputStream(file));
        setTotalByteCountInEDT((int)file.length());
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    public ProgressInputStream (@Nonnull final URL url)
      throws IOException
      {
        final URLConnection connection = url.openConnection();
        final int length = connection.getContentLength();
        final String mimeType = connection.getContentType();
        logger.finer(">>>> length from HTTP headers: %d", length);
        logger.finer(">>>> MIME type from HTTP headers: %s", mimeType);

        if (length > 0)
          {
            setTotalByteCountInEDT(length);
          }

        this.delegate = connection.getInputStream();
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    public ProgressInputStream (@Nonnull final InputStream delegate,
                                @Nonnegative final int size)
      {
        this.delegate = delegate;
        setTotalByteCountInEDT(size);
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    public ProgressInputStream (@Nonnull final InputStream delegate)
      {
        this.delegate = delegate;
      }

    /***************************************************************************
     *
     * {@inheritDoc}
     *
     **************************************************************************/
    @Override
    public long skip (long arg0)
      throws IOException
      {
        setTotalByteCountInEDT(getTotalByteCount() + (int)arg0);
        return delegate.skip(arg0);
      }

    /***************************************************************************
     *
     * {@inheritDoc}
     *
     **************************************************************************/
    @Override
    public synchronized void reset() 
      throws IOException
      {
        delegate.reset();
      }

    /***************************************************************************
     *
     * {@inheritDoc}
     *
     **************************************************************************/
    @Override
    public int read (byte[] buffer, int offset, int length)
      throws IOException
      {
        final int n = delegate.read(buffer, offset, length);
        setProcessedByteCountInEDT(getProcessedByteCount() + n);
        return n;
      }

    /***************************************************************************
     *
     * {@inheritDoc}
     *
     **************************************************************************/
    @Override
    public int read (byte[] arg0)
      throws IOException
      {
        final int n = delegate.read(arg0);
        setProcessedByteCountInEDT(getProcessedByteCount() + n);
        return n;
      }

    /***************************************************************************
     *
     * {@inheritDoc}
     *
     **************************************************************************/
    @Override
    public int read()
      throws IOException
      {
        final int n = delegate.read();
        setProcessedByteCountInEDT(getProcessedByteCount() + n);
        return n;
      }

    /***************************************************************************
     *
     * {@inheritDoc}
     *
     **************************************************************************/
    @Override
    public boolean markSupported()
      {
        return delegate.markSupported();
      }

    /***************************************************************************
     *
     * {@inheritDoc}
     *
     **************************************************************************/
    @Override
    public synchronized void mark (int arg0)
      {
        delegate.mark(arg0);
      }

    /***************************************************************************
     *
     * {@inheritDoc}
     *
     **************************************************************************/
    @Override
    public void close() 
      throws IOException
      {
        delegate.close();
      }

    /***************************************************************************
     *
     * {@inheritDoc}
     *
     **************************************************************************/
    @Override
    public int available() 
      throws IOException
      {
        return delegate.available();
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    protected void setProcessedByteCount (@Nonnegative final int processedByteCount)
      {
        int oldProcessedByteCount = this.processedByteCount;
        this.processedByteCount = processedByteCount;
        propertyChangeSupport.firePropertyChange(PROP_PROCESSED_BYTE_COUNT, oldProcessedByteCount, processedByteCount);
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    @Nonnegative
    public int getProcessedByteCount()
      {
        return processedByteCount;
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    protected void setTotalByteCount (@Nonnegative final int totalByteCount)
      {
        int oldTotalByteCount = this.totalByteCount;
        this.totalByteCount = totalByteCount;
        propertyChangeSupport.firePropertyChange(PROP_TOTAL_BYTE_COUNT, oldTotalByteCount, totalByteCount);
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    public int getTotalByteCount()
      {
        return totalByteCount;
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    public void addPropertyChangeListener (PropertyChangeListener listener)
      {
        propertyChangeSupport.addPropertyChangeListener(listener);
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    public void removePropertyChangeListener (PropertyChangeListener listener)
      {
        propertyChangeSupport.removePropertyChangeListener(listener);
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    private void setTotalByteCountInEDT (final int totalByteCount)
      {
        EventQueue.invokeLater(new Runnable()
          {
            public void run()
              {
                setTotalByteCount(totalByteCount);
              }
          });
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    private void setProcessedByteCountInEDT (final int processedByteCount)
      {
        EventQueue.invokeLater(new Runnable()
          {
            public void run()
              {
                setProcessedByteCount(processedByteCount);
              }
          });
      }
  }
